%-----------------------------------------------------------------------------------------
%-----------------------------------------------------------------------------------------
%-- b12_s40BS.erl
%--
%-- AUTHORS: 	 Tamarit-Emartinm
%-- DATE:        2013           
%-- PUBLISHED:   This is the (only) minimal slice of the program solving the ternary
%--              calculation problem implemented in the file ternary_ok.erl available at:
%-- 			 https://github.com/tamarit/edd/blob/master/examples/ternary
%--
%-- COPYLEFT:    This Erlang version is distributed in "Bencher: The Program Slicing 
%--              Benchmark Suite for Erlang" (Universitat Politècnica de València). 
%--              It is free of use and publicly available in the URL: 
%--              http://www.dsic.upv.es/~jsilva/slicing/bencher/
%--
%-- DESCRIPTION
%-- The program performs translations and operations with balanced ternary. Its inputs are
%-- two numbers in balanced ternary representation and a decimal number. It converts the 
%-- three numbers to the oposite representation and performs an operation with their 
%-- balanced ternary representation. The output is a tuple that contains the input numbers
%-- and the result in both representations.
%--
%-- ORIGINAL PROGRAM: Available at: 
%--    http://www.dsic.upv.es/~jsilva/slicing/bencher/programs/bench12.erl
%-- SLICING CRITERION: (40,BS)/(Node 73)    
%-----------------------------------------------------------------------------------------
%-----------------------------------------------------------------------------------------

-module(b12_s40BS).
-export([main/3]).
 
main(_,B,_) ->
    BT = to_ternary(B), 
    BS = to_string(BT),
    [undef,{BS,undef}|undef].

to_string(T) -> [to_char(X) || X <- T].

to_char(-1) -> $-;
to_char(0) -> $0;
to_char(_) -> $+.
   
to_ternary(N) when N > 0 ->
    to_ternary(N,[]);
to_ternary(N) ->
    neg(to_ternary(-N)).
 
to_ternary(0,Acc) ->
    Acc;
to_ternary(N,Acc) when N rem 3 == 0 ->
    to_ternary(N div 3, [0|Acc]);
to_ternary(N,Acc) when N rem 3 == 1 ->
    to_ternary(N div 3, [1|Acc]);
to_ternary(N,Acc) ->
    _ = to_ternary((N+1) div 3, [-1|Acc]).

neg(T) -> [ -H || H <- T].
