%-----------------------------------------------------------------------------------------
%-----------------------------------------------------------------------------------------
%-- b15_s65Shown.erl
%--
%-- AUTHORS:     Tamarit
%-- DATE:        2015           
%-- PUBLISHED:   This is the (only) minimal slice of the program solving the combinations
%--              and permutations problem implemented in Erlang available at: 
%--              http://rosettacode.org/wiki/Combinations_and_permutations#Erlang
%--
%-- COPYLEFT:    This Erlang version is distributed in "Bencher: The Program Slicing 
%--              Benchmark Suite for Erlang" (Universitat Politècnica de València). 
%--              It is free of use and publicly available in the URL: 
%--              http://www.dsic.upv.es/~jsilva/slicing/bencher/
%--
%-- DESCRIPTION
%-- The program implements the procedure for making permutations and combinations in an 
%-- interval of given numbers. The inputs are the first and last numbers of the 
%-- permutation interval and the first and last numbers of the combination interval. The 
%-- output will be a set of the permutations and combinations calculated.
%--
%-- ORIGINAL PROGRAM: Available at: 
%--    http://www.dsic.upv.es/~jsilva/slicing/bencher/programs/bench15.erl
%-- SLICING CRITERION: (65,Shown)/(Node 244)
%-----------------------------------------------------------------------------------------
%-----------------------------------------------------------------------------------------

-module(b15_s65Shown).
-export([main/4]).
 
perm(N, K) ->
    product(lists:seq(N - K + 1, N)).
 
comb(N, K) ->
    perm(N, K) div product(lists:seq(1, K)).
 
product(List) ->
    lists:foldl(fun(N, Acc) -> N * Acc end, 1, List).
 
main(PFrom,PTo,CFrom,CTo) ->
    IncremP = if (PTo - PFrom >= 10) -> (PTo-PFrom) div 10;
                true -> 1
              end,
    IncremC = if (CTo - CFrom >= 10) -> (CTo-CFrom) div 10;
                true -> 1
              end,
    _=[show_perm({N, N div 3}) || N <- lists:seq(PFrom, PTo, IncremP)],
    
    _=[show_comb({N, N div 3}) || N <- lists:seq(CFrom, CTo, IncremC)].
 
show_perm({N, K}) ->
    show_gen(N, K, undef, fun perm/2).
 
show_comb({N, K}) ->
    show_gen(N, K, undef, fun comb/2).
 
show_gen(N, K, _, Fun) ->
    funundef(undef,[undef, undef, undef, show_big(Fun(N, K), 40)|undef]). 
 
show_big(N, Limit) ->
    StrN = integer_to_list(N),
    case length(StrN) < Limit of
		true -> StrN;        
		_ -> 
            {Shown, _} = lists:split(Limit, StrN)
    end. 

funundef(_,_) -> undef.
