%-----------------------------------------------------------------------------------------
%-----------------------------------------------------------------------------------------
%-- b17_s54X.erl
%--
%-- AUTHORS:     Anonymous
%-- DATE:        2001
%-- PUBLISHED:   This is the (only) minimal slice of a program known as the SCAM Mug that
%--              appeared under the title “Which Lines do not affect x?” in a ceramic mug
%--              given to attendees of the First Source Code Analysis and Manipulation
%--              Workshop (SCAM), celebrated in Florence, Italy, on November 10th, 2001.
%--
%-- COPYLEFT:    This Erlang version is distributed in "Bencher: The Program Slicing 
%--              Benchmark Suite for Erlang" (Universitat Politècnica de València). 
%--              It is free of use and publicly available in the URL: 
%--              http://www.dsic.upv.es/~jsilva/slicing/bencher/              
%--
%-- DESCRIPTION
%-- The SCAM Mug Example, Predicates p and q, and function h depend only on their single
%-- parameter while functions f and g return (unknown) constant values. The key point in
%-- this code is that in any terminating execution the final value of X is independent of
%-- the case q(c) expression: if it is initially false, it remains false and thus X retains
%-- its initial value. If it is true, one or more times then X will have the value assigned
%-- at Line 45. In the latter case, it does not matter how often q(c) is true and thus the 
%-- assignment at Line 46 does not impact the value of x at Line 54. (Description taken 
%-- from "ORBS and the Limits of Static Slicing" and adapted to Erlang).
%--
%-- ORIGINAL PROGRAM: Available at: 
%--    http://www.dsic.upv.es/~jsilva/slicing/bencher/programs/bench17.erl
%-- SLICING CRITERION: (54,X)
%-----------------------------------------------------------------------------------------
%-----------------------------------------------------------------------------------------
-module(b17_s54X).
-export([mug/3]).

-spec mug(integer(),integer(),integer()) -> any().
mug(I, C, X) ->
  case p(I) of
    true ->
      {NewX,_} =
        case q(C) of
          true ->
            TempX = 10,
            {TempX,sliced};
          false ->
            {X,sliced}
      end,
      mug(sliced, sliced, NewX);
    _ ->
      funsliced(sliced, [X|sliced])
  end.

p(I) ->
  I < 10.

q(C) ->
  C < 20.

funsliced(_, _) ->
  sliced.
