%-----------------------------------------------------------------------------------------
%-----------------------------------------------------------------------------------------
%-- b1_s56Year.erl
%--
%-- AUTHORS:     Anonymous
%-- DATE:        2016           
%-- PUBLISHED:   This is one of the possible minimal slices of a program specially 
%--              developed to test compression and expansion of complex structures.
%--
%-- COPYLEFT:    This Erlang version is distributed in "Bencher: The Program Slicing 
%--              Benchmark Suite for Erlang" (Universitat Politècnica de València). 
%--              It is free of use and publicly available in the URL: 
%--              http://www.dsic.upv.es/~jsilva/slicing/bencher/
%--
%-- DESCRIPTION
%-- This benchmark consists in a program with a database of singers. It returns a set
%-- of information of each singer (Name,Age,Last Album Name, and Location and Year of 
%-- their next concert) by receiving a number between 1 and 6 as input.
%-- 
%-- ORIGINAL PROGRAM: Available at: 
%--    http://www.dsic.upv.es/~jsilva/slicing/bencher/programs/bench1.erl
%-- SLICING CRITERION: (56,Year)/(Node 414)
%-----------------------------------------------------------------------------------------
%-----------------------------------------------------------------------------------------
-module(b1_s56Year).
-export([main/1]).

main(Number)
    when
        Number > 0
        andalso
        Number < 7 ->
    Database =
        [[undef,
          undef,
          undef,
          {undef,
           [{undef,{undef,undef,undef}},
            {undef,{undef,undef,undef}},
            {undef,{undef,undef,undef}},
            {undef,{2017,undef,undef}}]}|
          undef],
         [undef,
          undef,
          undef,
          {undef,
           [{undef,{undef,undef,undef}},{undef,{undef,undef,undef}}]}|
          undef],
         [undef,
          undef,
          undef,
          {undef,
           [{undef,{undef,undef,undef}},
            {undef,{undef,undef,undef}},
            {undef,{undef,undef,undef}}]}|
          undef],
         [undef,
          undef,
          undef,
          {undef,
           [{undef,{undef,undef,undef}},
            {undef,{undef,undef,undef}},
            {undef,{2017,undef,undef}}]}|
          undef],
         [undef,
          undef,
          undef,
          {undef,
           [{undef,{undef,undef,undef}},
            {undef,{undef,undef,undef}},
            {undef,{2017,undef,undef}}]}|
          undef],
         [undef,
          undef,
          undef,
          {undef,
           [{undef,{undef,undef,undef}},
            {undef,{undef,undef,undef}},
            {undef,{undef,undef,undef}},
            {undef,{2017,undef,undef}}]}|
          undef]|
         undef],
    Artist = lists:nth(Number, Database),
    NextConcert = getNextConcert(Artist),
    Info = getConcertLocationAndYear(NextConcert),
    {_,Year} = Info.

getConcerts(Artist) ->
    [_,_,_,{_,Concerts}|_] = Artist,
    Concerts.

getNextConcert(Artist) ->
    CurrentDate = {2017,undef,undef},
    Concerts = getConcerts(Artist),
    case getNext(Concerts, CurrentDate, undef) of
        empty ->
            "No future concerts planned";
        Concert ->
            Concert
    end.

getNext([], _, NextConcert) ->
    NextConcert;
getNext([Concert|Concerts], Current, _) ->
    {YearA,_,_} = Current,
    {_,Date} = Concert,
    {YearC,_,_} = Date,
    case {YearA,YearC,undef,undef,undef,undef} of
        {Y,Y,_,_,DA,DC} when DC >= DA ->
            getNext(Concerts, undef, Concert);
        _ ->
            getNext(Concerts, Current, empty)
    end.

getConcertLocationAndYear("No future concerts planned") ->
    {undef,"Not planned yet"};
getConcertLocationAndYear(Concert) ->
    {_,Date} = Concert,
    {Year,_,_} = Date,
    _ = {undef,Year}.
